﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.IO;
using System.IO.Compression;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Windows.Forms;

namespace CoD2SM
{
    public partial class MainForm : Form
    {
        Dictionary<string, string> stockFileList = new Dictionary<string, string>();
        string mapPacksPath = Path.Combine(Application.StartupPath, "map_packs");

        public MainForm()
        {
            InitializeComponent();
            this.Icon = (Icon)Properties.Resources.CoD2blue32;
            SilDev.Source.IncludeAssemblies();
        }

        private void MainForm_Load(object sender, EventArgs e)
        {
            SilDev.IrrKlangLib.Play(SilDev.Source.GetFilePath(SilDev.Source.files[0]), true, 35);
            string stockListPath = Path.Combine(SilDev.Source.GetPath(), "_stock");
            try
            {
                foreach (var list in Directory.GetFiles(stockListPath, "*.txt"))
                {
                    using (StreamReader readIgnoreList = new StreamReader(list))
                    {
                        string listContent = readIgnoreList.ReadToEnd();
                        stockFileList.Add(Path.GetFileNameWithoutExtension(list), listContent);
                    }
                }
            }
            catch
            {
                MessageBox.Show("The stock filters couldn't be loaded. Make sure there is enough disk space for temporary files.", "Warning", MessageBoxButtons.OK, MessageBoxIcon.Warning);
            }
        }

        private void btnMusic_Click(object sender, EventArgs e)
        {
            if (!SilDev.IrrKlangLib.MusicIsRunning)
                SilDev.IrrKlangLib.Play(SilDev.Source.GetFilePath(SilDev.Source.files[0]), true, 35);
            else
                SilDev.IrrKlangLib.Stop();
        }

        private void btnMusic_MouseEnter(object sender, EventArgs e)
        {
            btnMusic.BackgroundImage = (Image)Properties.Resources._BACK_32p_05;
        }

        private void btnMusic_MouseLeave(object sender, EventArgs e)
        {
            btnMusic.BackgroundImage = (Image)Properties.Resources._BACK_32p_01;
        }

        private void btnHelp_Click(object sender, EventArgs e)
        {
            MessageBox.Show("Comming soon!", "Help", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
        }

        private void btnHelp_MouseEnter(object sender, EventArgs e)
        {
            btnHelp.BackgroundImage = (Image)Properties.Resources._BACK_32p_05;
        }

        private void btnHelp_MouseLeave(object sender, EventArgs e)
        {
            btnHelp.BackgroundImage = (Image)Properties.Resources._BACK_32p_01;
        }

        private void btn1_Click(object sender, EventArgs e)
        {
            if (btn1.Text == "START")
            {
                string stockListPath = Path.Combine(SilDev.Source.GetPath(), "_stock");

                currentMap.ForeColor = Color.LightBlue;
                currentFile.ForeColor = Color.LightBlue;
                btn1.Enabled = false;
                bwMapsSorter.RunWorkerAsync();
            }
            else
            {
                Process.Start(mapPacksPath);
                this.Close();
            }
        }

        private void btn1_MouseEnter(object sender, EventArgs e)
        {
            btn1.ForeColor = Color.Gainsboro;
            btn1.BackgroundImage = (Image)Properties.Resources._BACK_23p_05;
        }

        private void btn1_MouseLeave(object sender, EventArgs e)
        {
            btn1.ForeColor = Color.Gainsboro;
            btn1.BackgroundImage = (Image)Properties.Resources._BACK_23p_01;
        }

        private void bwMapsSorter_DoWork(object sender, DoWorkEventArgs e)
        {
            string mainPath = Path.Combine(Application.StartupPath, "main");
            string mapsPath = Path.Combine(Application.StartupPath, "main\\maps\\mp");
            Dictionary<string, string> newSubDir = new Dictionary<string, string>();
            if (Directory.Exists(mapsPath))
            {
                // Get the map files and start the work
                foreach (var map in Directory.GetFiles(mapsPath, "*.d3dbsp"))
                {
                    string mapName = Path.GetFileNameWithoutExtension(map);
                    bwMapsSorter.ReportProgress(0, string.Format("Map: \"{0}\"", mapName));
                    string newDir = Path.Combine(mapPacksPath, mapName);
                    if (Directory.Exists(newDir))
                        Directory.Delete(newDir, true);

                    newSubDir.Clear();
                    newSubDir.Add("maps", Path.Combine(newDir, "maps\\mp"));

                    // Copy the map files
                    bwMapsSorter.ReportProgress(0, string.Format("File: \"{0}\"", Path.GetFileName(map)));
                    if (!Directory.Exists(newSubDir["maps"]))
                        Directory.CreateDirectory(newSubDir["maps"]);

                    File.Copy(map, Path.Combine(newSubDir["maps"], Path.GetFileName(map)));
                    string secondMapFile = Path.Combine(Application.StartupPath, string.Format("main\\maps\\mp\\{0}.d3dprt", mapName));
                    if (File.Exists(secondMapFile))
                    {
                        bwMapsSorter.ReportProgress(0, string.Format("File: \"{0}\"", Path.GetFileName(secondMapFile)));
                        File.Copy(secondMapFile, Path.Combine(newSubDir["maps"], Path.GetFileName(secondMapFile)));
                    }

                    // Read the full content of the map file and save it
                    StreamReader readMap = new StreamReader(map);
                    string mapContentStr = readMap.ReadToEnd();
                    readMap.Close();
                    string[] mapContentArr = mapContentStr.Split((char)10);

                    // All materials are in the first 2 lines - to be sure, I take the first 4 lines to a new array
                    string[] mapContentMatsArr = mapContentArr.Take(3).ToArray();
                    List<string> mcmafilter = new List<string>();
                    mcmafilter.Add(char.ToString((char)1));
                    mcmafilter.Add(char.ToString((char)2));
                    mcmafilter.Add(char.ToString((char)3));
                    mcmafilter.Add(char.ToString((char)8));
                    mcmafilter.Add(char.ToString((char)32));
                    mcmafilter.Add(char.ToString((char)40));
                    for (int i = 0; i < mapContentMatsArr.Length; i++)
                        foreach (var f in mcmafilter) // Many filters to find the materials eaysier and faster later.
                            mapContentMatsArr[i] = mapContentMatsArr[i].Replace(f, char.ToString((char)0));

                    // "classname" "worldspawn" is a very good point to take a new array from there up to the end for the xmodels part later
                    string[] mapContentXmArr = new string[255];
                    for (int i = 5; i < mapContentArr.Length; i++)
                    {
                        if (mapContentArr[i].Contains("\"classname\" \"worldspawn\""))
                        {
                            mapContentXmArr = mapContentArr.Skip(i).Take(mapContentArr.Length - 1).ToArray();
                            break;
                        }
                    }

                    // Find and copy materials by the map content
                    newSubDir.Add("mats", Path.Combine(newDir, "materials"));
                    string matsPath = Path.Combine(Application.StartupPath, "main\\materials");
                    if (Directory.Exists(matsPath) && mapContentMatsArr.Length > 0)
                    {
                        List<string> alreadyFound = new List<string>();
                        foreach (var line in mapContentMatsArr)
                        {
                            foreach (var str in line.Split((char)0))
                            {
                                if (str.Length > 1 && str.ToCharArray().All(c => Char.IsLetter(c) || Char.IsNumber(c) || c == (char)95))
                                {
                                    string file = str;
                                    if (listContains(alreadyFound, file) || isStockFile(file, "materials"))
                                        continue;

                                    string fileLoc = Path.Combine(matsPath, file);
                                    string fileCopyLoc = Path.Combine(newSubDir["mats"], file);
                                    if (File.Exists(fileLoc) && !File.Exists(fileCopyLoc))
                                    {
                                        bwMapsSorter.ReportProgress(0, string.Format("File: \"{0}\"", file));
                                        if (!Directory.Exists(newSubDir["mats"]))
                                            Directory.CreateDirectory(newSubDir["mats"]);

                                        File.Copy(fileLoc, fileCopyLoc);
                                    }
                                    alreadyFound.Add(file);
                                }
                            }
                        }
                    }

                    // Find and copy xmodels by the map content
                    newSubDir.Add("xm", Path.Combine(newDir, "xmodel"));
                    string xmPath = Path.Combine(Application.StartupPath, "main\\xmodel");
                    if (Directory.Exists(xmPath) && mapContentXmArr.Length > 0)
                    {
                        List<string> alreadyFound = new List<string>();
                        foreach (var line in mapContentXmArr)
                        {
                            var file = Regex.Match(line, "\"xmodel/(.+?)\"").Groups[1].Value;
                            if (!string.IsNullOrEmpty(file))
                            {
                                if (listContains(alreadyFound, file) || isStockFile(file, "xmodel"))
                                    continue;

                                string fileLoc = Path.Combine(xmPath, file);
                                string fileCopyLoc = Path.Combine(newSubDir["xm"], file);
                                if (File.Exists(fileLoc) && !File.Exists(fileCopyLoc))
                                {
                                    bwMapsSorter.ReportProgress(0, string.Format("File: \"{0}\"", file));
                                    if (!Directory.Exists(newSubDir["xm"]))
                                        Directory.CreateDirectory(newSubDir["xm"]);

                                    File.Copy(fileLoc, fileCopyLoc);
                                }
                                alreadyFound.Add(file);
                            }
                        }
                        alreadyFound.Clear();
                    }

                    // Find and copy all map scripts
                    string mapGsc = Path.Combine(mapsPath, string.Format("{0}.gsc", mapName));
                    if (File.Exists(mapGsc))
                    {
                        // Read the mapname.gsc to find more gsc scripts
                        List<string> alreadyFound = new List<string>();
                        string mapGscCopyLoc = Path.Combine(newSubDir["maps"], Path.GetFileName(mapGsc));
                        if (!File.Exists(mapGscCopyLoc))
                        {
                            if (!Directory.Exists(newSubDir["maps"]))
                                Directory.CreateDirectory(newSubDir["maps"]);

                            File.Copy(mapGsc, mapGscCopyLoc);
                            alreadyFound.Add(Path.GetFileName(mapGsc));
                        }
                        foreach (var line in File.ReadLines(mapGsc))
                        {
                            var file = (!string.IsNullOrEmpty(Regex.Match(line, "#include(.+?);").Groups[1].Value) ? Regex.Match(line, "#include(.+?);").Groups[1].Value : Regex.Match(line, "(.+?)::").Groups[1].Value).Replace(" ", string.Empty);
                            if (!string.IsNullOrEmpty(file))
                            {
                                string[] nameFilter = file.Split('\\');
                                string gscSubFolder = string.Empty;
                                if (nameFilter.Length > 3)
                                {
                                    for (int i = 2; i < nameFilter.Length - 1; i++)
                                        gscSubFolder += (nameFilter[i] + "\\");

                                    gscSubFolder = gscSubFolder.Substring(0, (gscSubFolder.Length - 1));
                                }
                                file = nameFilter[nameFilter.Length - 1] + ".gsc";
                                if (listContains(alreadyFound, file) || file.Contains((char)43) || file.Contains((char)61) || file.Contains((char)94) || isStockFile(file, "maps"))
                                    continue;

                                string fileLoc = string.Empty;
                                string fileCopyLoc = string.Empty;
                                if (!string.IsNullOrWhiteSpace(gscSubFolder))
                                {
                                    fileLoc = Path.Combine(mapsPath, gscSubFolder);
                                    fileLoc = Path.Combine(fileLoc, file);
                                    fileCopyLoc = Path.Combine(newSubDir["maps"], gscSubFolder);
                                    fileCopyLoc = Path.Combine(fileCopyLoc, file);
                                }
                                else
                                {
                                    fileLoc = Path.Combine(mapsPath, file);
                                    fileCopyLoc = Path.Combine(newSubDir["maps"], file);
                                }
                                if (File.Exists(fileLoc) && !File.Exists(fileCopyLoc))
                                {
                                    bwMapsSorter.ReportProgress(0, string.Format("File: \"{0}\"", file));
                                    string copyLocDir = (!string.IsNullOrWhiteSpace(gscSubFolder) ? Path.Combine(newSubDir["maps"], gscSubFolder) : newSubDir["maps"]);
                                    if (!Directory.Exists(copyLocDir))
                                        Directory.CreateDirectory(copyLocDir);

                                    File.Copy(fileLoc, fileCopyLoc);
                                }
                            }
                        }
                    }

                    // Find and copy the arena file
                    newSubDir.Add("mp", Path.Combine(newDir, "mp"));
                    string mpPath = Path.Combine(mainPath, "mp");
                    if (Directory.Exists(mpPath))
                    {
                        string file = Path.GetFileName(string.Format("{0}.arena", mapName));
                        string fileLoc = Path.Combine(mpPath, file);
                        string fileCopyLoc = Path.Combine(newSubDir["mp"], file);
                        if (File.Exists(fileLoc) && !File.Exists(fileCopyLoc) && !isStockFile(file, "mp"))
                        {
                            bwMapsSorter.ReportProgress(0, string.Format("File: \"{0}\"", file));
                            if (!Directory.Exists(Path.GetDirectoryName(fileCopyLoc)))
                                Directory.CreateDirectory(Path.GetDirectoryName(fileCopyLoc));

                            File.Copy(fileLoc, fileCopyLoc);
                        }
                    }

                    // Find and copy the soundaliases file
                    newSubDir.Add("snda", Path.Combine(newDir, "soundaliases"));
                    string sndaPath = Path.Combine(mainPath, "soundaliases");
                    if (Directory.Exists(sndaPath))
                    {
                        string file = Path.GetFileName(string.Format("{0}.csv", mapName));
                        string fileLoc = Path.Combine(sndaPath, file);
                        string fileCopyLoc = Path.Combine(newSubDir["snda"], file);
                        if (File.Exists(fileLoc) && !File.Exists(fileCopyLoc) && !isStockFile(file, "soundaliases"))
                        {
                            bwMapsSorter.ReportProgress(0, string.Format("File: \"{0}\"", file));
                            if (!Directory.Exists(Path.GetDirectoryName(fileCopyLoc)))
                                Directory.CreateDirectory(Path.GetDirectoryName(fileCopyLoc));

                            File.Copy(fileLoc, fileCopyLoc);
                        }
                    }

                    // Read the map scripts to find fx's, localizedstrings, shaders, shellshocks, soundaliases, ui's, weapons, xmodels and copy them to the new map directory
                    if (Directory.Exists(newSubDir["maps"]))
                    {
                        List<string> wpnFound = new List<string>();
                        List<string> fxFound = new List<string>();
                        List<string> shaderFound = new List<string>();
                        List<string> shockFound = new List<string>();
                        List<string> uiFound = new List<string>();
                        List<string> xmFound = new List<string>();
                        List<string> localsFound = new List<string>();
                        List<string> sndFound = new List<string>();
                        foreach (var gsc in Directory.GetFiles(newSubDir["maps"], "*.gsc", SearchOption.AllDirectories))
                        {
                            foreach (var uline in File.ReadAllText(gsc).Split((char)59))
                            {
                                if (!string.IsNullOrWhiteSpace(uline))
                                {
                                    var line = uline.Replace(" ", string.Empty).ToLower();
                                    if (line.Contains("precacheitem"))
                                    {
                                        string file = Regex.Match(line, "precacheitem[(.*)]\"(.+?)\"").Groups[1].Value;
                                        if (!string.IsNullOrEmpty(file) && !listContains(wpnFound, file))
                                            wpnFound.Add(file);
                                    }
                                    if (line.Contains("loadfx"))
                                    {
                                        string file = Regex.Match(line, "loadfx[(.*)]\"fx/(.+?)\"").Groups[1].Value;
                                        if (!string.IsNullOrEmpty(file) && !listContains(fxFound, file))
                                            fxFound.Add(file);
                                    }
                                    if (line.Contains("precacheshader"))
                                    {
                                        string file = Regex.Match(line, "precacheshader[(.*)]\"(.+?)\"").Groups[1].Value;
                                        if (!string.IsNullOrEmpty(file) && !listContains(shaderFound, file))
                                            shaderFound.Add(file);
                                    }
                                    if (line.Contains("precacheshellshock"))
                                    {
                                        string file = Regex.Match(line, "precacheshellshock[(.*)]\"(.+?)\"").Groups[1].Value;
                                        if (!string.IsNullOrEmpty(file) && !listContains(shockFound, file))
                                            shockFound.Add(file);
                                    }
                                    if (line.Contains("precachemenu"))
                                    {
                                        string file = Regex.Match(line, "precachemenu[(.*)]\"(.+?)\"").Groups[1].Value;
                                        if (!string.IsNullOrEmpty(file) && !listContains(uiFound, file))
                                            uiFound.Add(file);
                                    }
                                    if (line.Contains("precachemodel"))
                                    {
                                        string file = Regex.Match(line, "precachemodel[(.*)]\"xmodel/(.+?)\"").Groups[1].Value;
                                        if (!string.IsNullOrEmpty(file) && !listContains(xmFound, file))
                                            xmFound.Add(file);
                                    }
                                    if (line.Contains("precachestring"))
                                    {
                                        string file = Regex.Match(line, "precachestring[(.*)]&\"(.+?)\"").Groups[1].Value;
                                        if (!string.IsNullOrEmpty(file) && !listContains(localsFound, file))
                                            localsFound.Add(file);
                                    }
                                    if (line.Contains("iprintlnbold"))
                                    {
                                        string file = Regex.Match(line, "iprintlnbold[(.*)]&\"(.+?)\"").Groups[1].Value;
                                        if (!string.IsNullOrEmpty(file) && !listContains(localsFound, file))
                                            localsFound.Add(file);
                                    }
                                    else
                                    {
                                        if (line.Contains("iprintln"))
                                        {
                                            string file = Regex.Match(line, "iprintln[(.*)]&\"(.+?)\"").Groups[1].Value;
                                            if (!string.IsNullOrEmpty(file) && !listContains(localsFound, file))
                                                localsFound.Add(file);
                                        }
                                    }
                                }
                            }
                        }
                        if (wpnFound.Count > 0)
                        {
                            newSubDir.Add("wpns", Path.Combine(newDir, "weapons\\mp"));
                            foreach (var wpn in wpnFound)
                            {
                                string fileLoc = Path.Combine(mainPath, string.Format("weapons\\mp\\{0}", wpn.Replace("/", "\\")));
                                string fileCopyLoc = Path.Combine(newSubDir["wpns"], wpn.Replace("/", "\\"));
                                if (File.Exists(fileLoc) && !File.Exists(fileCopyLoc) && !isStockFile(Path.GetFileName(fileLoc), "weapons"))
                                {
                                    bwMapsSorter.ReportProgress(0, string.Format("File: \"{0}\"", Path.GetFileName(fileCopyLoc)));
                                    if (!Directory.Exists(Path.GetDirectoryName(fileCopyLoc)))
                                        Directory.CreateDirectory(Path.GetDirectoryName(fileCopyLoc));

                                    File.Copy(fileLoc, fileCopyLoc);
                                }
                                if (Directory.Exists(newSubDir["wpns"]))
                                {
                                    foreach (var wpns in Directory.GetFiles(newSubDir["wpns"]))
                                    {
                                        using (StreamReader stream = new StreamReader(wpns))
                                        {
                                            string wpnContentStr = stream.ReadToEnd().Replace("displayName\\", "displayName/").Replace("gunModel\\", "gunModel/").Replace("fireSound\\", "fireSound/").Replace(" ", string.Empty);
                                            foreach (var uline in wpnContentStr.Split((char)92))
                                            {
                                                if (!string.IsNullOrWhiteSpace(uline))
                                                {
                                                    var line = string.Format("{0}/", uline.ToLower());
                                                    if (line.Contains("displayname/"))
                                                    {
                                                        string file = Regex.Match(line, "displayName/(.+?)/").Groups[1].Value;
                                                        if (!string.IsNullOrEmpty(file) && !listContains(localsFound, file))
                                                            localsFound.Add(file);
                                                    }
                                                    if (line.Contains("fx/"))
                                                    {
                                                        string file = Regex.Match(line, "fx/(.+?)/").Groups[1].Value;
                                                        if (!string.IsNullOrEmpty(file) && !listContains(fxFound, file))
                                                            fxFound.Add(file);
                                                    }
                                                    if (line.Contains("gunmodel/"))
                                                    {
                                                        string file = Regex.Match(line, "gunmodel/(.+?)/").Groups[1].Value;
                                                        if (!string.IsNullOrEmpty(file) && !listContains(fxFound, file))
                                                            xmFound.Add(file);
                                                    }
                                                    if (line.Contains("xmodel/"))
                                                    {
                                                        string file = Regex.Match(line, "xmodel/(.+?)/").Groups[1].Value;
                                                        if (!string.IsNullOrEmpty(file) && !listContains(xmFound, file))
                                                            xmFound.Add(file);
                                                    }
                                                    if (line.Contains("firesound/"))
                                                    {
                                                        string file = Regex.Match(line, "firesound/(.+?)/").Groups[1].Value;
                                                        if (!string.IsNullOrEmpty(file) && !listContains(xmFound, file))
                                                            sndFound.Add(file);
                                                    }
                                                }
                                            }
                                        }
                                    }
                                }
                            }
                        }
                        if (fxFound.Count > 0)
                        {
                            newSubDir.Add("fx", Path.Combine(newDir, "fx"));
                            foreach (var fx in fxFound)
                            {
                                string fileLoc = Path.Combine(mainPath, string.Format("fx\\{0}", fx.Replace("/", "\\")));
                                string fileCopyLoc = Path.Combine(newSubDir["fx"], fx.Replace("/", "\\"));
                                if (File.Exists(fileLoc) && !File.Exists(fileCopyLoc) && !isStockFile(Path.GetFileName(fileLoc), "fx"))
                                {
                                    bwMapsSorter.ReportProgress(0, string.Format("File: \"{0}\"", Path.GetFileName(fileCopyLoc)));
                                    if (!Directory.Exists(Path.GetDirectoryName(fileCopyLoc)))
                                        Directory.CreateDirectory(Path.GetDirectoryName(fileCopyLoc));

                                    File.Copy(fileLoc, fileCopyLoc);
                                }
                            }
                        }
                        if (shaderFound.Count > 0)
                        {
                            foreach (var shader in shaderFound)
                            {
                                string fileLoc = Path.Combine(mainPath, string.Format("materials\\{0}", shader.Replace("/", "\\")));
                                string fileCopyLoc = Path.Combine(newSubDir["mats"], shader.Replace("/", "\\"));
                                if (File.Exists(fileLoc) && !File.Exists(fileCopyLoc) && !isStockFile(Path.GetFileName(fileLoc), "materials"))
                                {
                                    bwMapsSorter.ReportProgress(0, string.Format("File: \"{0}\"", Path.GetFileName(fileCopyLoc)));
                                    if (!Directory.Exists(Path.GetDirectoryName(fileCopyLoc)))
                                        Directory.CreateDirectory(Path.GetDirectoryName(fileCopyLoc));

                                    File.Copy(fileLoc, fileCopyLoc);
                                }
                            }
                        }
                        if (shockFound.Count > 0)
                        {
                            newSubDir.Add("shock", Path.Combine(newDir, "shock"));
                            foreach (var shock in shockFound)
                            {
                                string fileLoc = Path.Combine(mainPath, string.Format("shock\\{0}", shock.Replace("/", "\\")));
                                string fileCopyLoc = Path.Combine(newSubDir["shock"], shock.Replace("/", "\\"));
                                if (File.Exists(fileLoc) && !File.Exists(fileCopyLoc) && !isStockFile(Path.GetFileName(fileLoc), "shock"))
                                {
                                    bwMapsSorter.ReportProgress(0, string.Format("File: \"{0}\"", Path.GetFileName(fileCopyLoc)));
                                    if (!Directory.Exists(Path.GetDirectoryName(fileCopyLoc)))
                                        Directory.CreateDirectory(Path.GetDirectoryName(fileCopyLoc));

                                    File.Copy(fileLoc, fileCopyLoc);
                                }
                            }
                        }
                        if (uiFound.Count > 0)
                        {
                            newSubDir.Add("ui", Path.Combine(newDir, "ui_mp\\scriptmenus"));
                            foreach (var ui in uiFound)
                            {
                                string fileLoc = Path.Combine(mainPath, string.Format("ui_mp\\scriptmenus\\{0}.menu", ui.Replace("/", "\\")));
                                string fileCopyLoc = Path.Combine(newSubDir["ui"], string.Format("{0}.menu", ui.Replace("/", "\\")));
                                string fileName = Path.GetFileName(fileLoc);
                                if (File.Exists(fileLoc) && !File.Exists(fileCopyLoc) && !isStockFile(fileName, "ui_mp"))
                                {
                                    bwMapsSorter.ReportProgress(0, string.Format("File: \"{0}\"", fileName));
                                    if (!Directory.Exists(Path.GetDirectoryName(fileCopyLoc)))
                                        Directory.CreateDirectory(Path.GetDirectoryName(fileCopyLoc));

                                    File.Copy(fileLoc, fileCopyLoc);
                                }
                            }
                        }
                        if (xmFound.Count > 0)
                        {
                            foreach (var xm in xmFound)
                            {
                                string fileLoc = Path.Combine(mainPath, string.Format("xmodel\\{0}", xm.Replace("/", "\\")));
                                string fileCopyLoc = Path.Combine(newSubDir["xm"], xm.Replace("/", "\\"));
                                string fileName = Path.GetFileName(fileLoc);
                                if (File.Exists(fileLoc) && !File.Exists(fileCopyLoc) && !isStockFile(fileName, "xmodel"))
                                {
                                    bwMapsSorter.ReportProgress(0, string.Format("File: \"{0}\"", fileName));
                                    if (!Directory.Exists(Path.GetDirectoryName(fileCopyLoc)))
                                        Directory.CreateDirectory(Path.GetDirectoryName(fileCopyLoc));

                                    File.Copy(fileLoc, fileCopyLoc);
                                }
                            }
                        }
                        if (localsFound.Count > 0)
                        {
                            string localsPath = Path.Combine(Application.StartupPath, "main\\localizedstrings");
                            if (Directory.Exists(localsPath))
                            {
                                newSubDir.Add("locals", Path.Combine(newDir, "localizedstrings"));
                                foreach (var str in localsFound)
                                {
                                    string[] local = str.Split((char)95);
                                    if (local.Length > 0)
                                    {
                                        if (!string.IsNullOrWhiteSpace(local[0]))
                                        {
                                            foreach (var file in Directory.GetFiles(localsPath, "*.str"))
                                            {
                                                if (Path.GetFileName(file).Contains(local[0]))
                                                {
                                                    string fileCopyLoc = Path.Combine(newSubDir["locals"], Path.GetFileName(file));
                                                    if (File.Exists(file) && !File.Exists(fileCopyLoc))
                                                    {
                                                        bwMapsSorter.ReportProgress(0, string.Format("File: \"{0}\"", file));
                                                        if (!Directory.Exists(newSubDir["locals"]))
                                                            Directory.CreateDirectory(newSubDir["locals"]);

                                                        File.Copy(file, fileCopyLoc);
                                                    }
                                                }
                                            }
                                        }
                                    }
                                }
                            }
                        }
                        if (sndFound.Count > 0)
                        {
                            if (Directory.Exists(sndaPath))
                            {
                                foreach (var snda in Directory.GetFiles(sndaPath, "*.csv"))
                                {
                                    using (StreamReader stream = new StreamReader(snda))
                                    {
                                        string sndaContentStr = stream.ReadToEnd();
                                        foreach (var snd in sndFound)
                                        {
                                            if (sndaContentStr.Contains(snd))
                                            {
                                                string file = Path.GetFileName(snda);
                                                string fileCopyLoc = Path.Combine(newSubDir["snda"], Path.GetFileName(file));
                                                if (File.Exists(snda) && !File.Exists(fileCopyLoc) && !isStockFile(file, "soundaliases"))
                                                {
                                                    bwMapsSorter.ReportProgress(0, string.Format("File: \"{0}\"", file));
                                                    if (!Directory.Exists(newSubDir["snda"]))
                                                        Directory.CreateDirectory(newSubDir["snda"]);

                                                    File.Copy(snda, fileCopyLoc);
                                                }
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }

                    // Find and copy xmodelparts and xmodelsurfs by the xmodels content
                    string xmpPath = Path.Combine(Application.StartupPath, "main\\xmodelparts");
                    string xmsPath = Path.Combine(Application.StartupPath, "main\\xmodelsurfs");
                    if (Directory.Exists(newSubDir["xm"]) && Directory.Exists(xmpPath) && Directory.Exists(xmsPath))
                    {
                        newSubDir.Add("xmp", Path.Combine(newDir, "xmodelparts"));
                        newSubDir.Add("xms", Path.Combine(newDir, "xmodelsurfs"));
                        foreach (var xm in Directory.GetFiles(newSubDir["xm"]))
                        {
                            List<string> alreadyFound = new List<string>();
                            using (StreamReader readXm = new StreamReader(xm))
                            {
                                string xmContentStr = readXm.ReadToEnd();
                                foreach (var str in xmContentStr.Split((char)0))
                                {
                                    if (str.Length > 1 && str.ToCharArray().All(c => Char.IsLetter(c) || Char.IsNumber(c) || c == (char)95))
                                    {
                                        string file = str;
                                        if (listContains(alreadyFound, file) || isStockFile(file, "xmodelparts") || isStockFile(file, "xmodelsurfs"))
                                            continue;

                                        string fileLoc = Path.Combine(xmpPath, file);
                                        string fileCopyLoc = Path.Combine(newSubDir["xmp"], file);
                                        if (File.Exists(fileLoc) && !File.Exists(fileCopyLoc))
                                        {
                                            bwMapsSorter.ReportProgress(0, string.Format("File: \"{0}\"", file));
                                            if (!Directory.Exists(newSubDir["xmp"]))
                                                Directory.CreateDirectory(newSubDir["xmp"]);

                                            File.Copy(fileLoc, fileCopyLoc);
                                        }
                                        fileLoc = Path.Combine(xmsPath, file);
                                        fileCopyLoc = Path.Combine(newSubDir["xms"], file);
                                        if (File.Exists(fileLoc) && !File.Exists(fileCopyLoc))
                                        {
                                            bwMapsSorter.ReportProgress(0, string.Format("File: \"{0}\"", file));
                                            if (!Directory.Exists(newSubDir["xms"]))
                                                Directory.CreateDirectory(newSubDir["xms"]);

                                            File.Copy(fileLoc, fileCopyLoc);
                                        }
                                        alreadyFound.Add(file);
                                    }
                                }
                            }
                        }
                    }

                    // Find and copy materials by the xmodel content
                    if (Directory.Exists(matsPath) && Directory.Exists(newSubDir["xm"]))
                    {
                        List<string> alreadyFound = new List<string>();
                        foreach (var xm in Directory.GetFiles(newSubDir["xm"]))
                        {
                            using (StreamReader stream = new StreamReader(xm))
                            {
                                string xmContentStr = stream.ReadToEnd();
                                foreach (var str in xmContentStr.Split((char)0))
                                {
                                    if (str.Length > 1 && str.ToCharArray().All(c => Char.IsLetter(c) || Char.IsNumber(c) || c == (char)95))
                                    {
                                        string file = str;
                                        if (listContains(alreadyFound, file) || isStockFile(file, "materials"))
                                            continue;

                                        string fileLoc = Path.Combine(matsPath, file);
                                        string fileCopyLoc = Path.Combine(newSubDir["mats"], file);
                                        if (File.Exists(fileLoc) && !File.Exists(fileCopyLoc))
                                        {
                                            bwMapsSorter.ReportProgress(0, string.Format("File: \"{0}\"", file));
                                            if (!Directory.Exists(newSubDir["mats"]))
                                                Directory.CreateDirectory(newSubDir["mats"]);

                                            File.Copy(fileLoc, fileCopyLoc);
                                        }
                                        alreadyFound.Add(file);
                                    }
                                }
                            }
                        }
                    }

                    // Find and copy the loadingscreen
                    string loadScreen = Path.Combine(Application.StartupPath, string.Format("main\\maps\\mp\\{0}.csv", mapName));
                    if (File.Exists(loadScreen))
                    {
                        File.Copy(loadScreen, Path.Combine(newSubDir["maps"], string.Format("{0}.csv", mapName)));
                        foreach (var line in File.ReadAllLines(loadScreen))
                        {
                            if (line.Contains("levelBriefing"))
                            {
                                string file = line.Replace("levelBriefing,", string.Empty).Replace(" ", string.Empty);
                                if (!string.IsNullOrWhiteSpace(file))
                                {
                                    if (isStockFile(file, "materials"))
                                        continue;

                                    string fileLoc = Path.Combine(matsPath, file);
                                    string fileCopyLoc = Path.Combine(newSubDir["mats"], file);
                                    if (File.Exists(fileLoc) && !File.Exists(fileCopyLoc))
                                    {
                                        bwMapsSorter.ReportProgress(0, string.Format("File: \"{0}\"", file));
                                        if (!Directory.Exists(newSubDir["mats"]))
                                            Directory.CreateDirectory(newSubDir["mats"]);

                                        File.Copy(fileLoc, fileCopyLoc);
                                    }
                                }
                            }
                        }
                    }

                    // Find and copy all images by the materials
                    string imgsPath = Path.Combine(Application.StartupPath, "main\\images");
                    if (Directory.Exists(imgsPath) && Directory.Exists(newSubDir["mats"]))
                    {
                        newSubDir.Add("imgs", Path.Combine(newDir, "images"));
                        List<string> alreadyFound = new List<string>();
                        foreach (var mats in Directory.GetFiles(newSubDir["mats"]))
                        {
                            using (StreamReader stream = new StreamReader(mats))
                            {
                                string matContentStr = stream.ReadToEnd();
                                foreach (var str in matContentStr.Split((char)0))
                                {
                                    if (str.Length > 1 && str.ToCharArray().All(c => Char.IsLetter(c) || Char.IsNumber(c) || c == (char)36 || c == (char)38 || c == (char)43 || c == (char)45 || c == (char)46 || c == (char)64 || c == (char)95 || c == (char)124 || c == (char)126))
                                    {
                                        string file = string.Format("{0}.iwi", str);
                                        if (listContains(alreadyFound, file) || isStockFile(file, "images"))
                                            continue;

                                        string fileLoc = Path.Combine(imgsPath, file);
                                        string fileCopyLoc = Path.Combine(newSubDir["imgs"], file);
                                        if (File.Exists(fileLoc) && !File.Exists(fileCopyLoc))
                                        {
                                            bwMapsSorter.ReportProgress(0, string.Format("File: \"{0}\"", file));
                                            if (!Directory.Exists(newSubDir["imgs"]))
                                                Directory.CreateDirectory(newSubDir["imgs"]);

                                            File.Copy(fileLoc, fileCopyLoc);
                                        }
                                        alreadyFound.Add(file);
                                    }
                                }
                            }
                        }
                    }

                    // Find and copy all sounds by the soundaliases
                    string sndPath = Path.Combine(Application.StartupPath, "main\\sound");
                    if (Directory.Exists(sndPath) && Directory.Exists(newSubDir["snda"]))
                    {
                        newSubDir.Add("snd", Path.Combine(newDir, "sound"));
                        List<string> alreadyFound = new List<string>();
                        foreach (var sndas in Directory.GetFiles(newSubDir["snda"], "*.csv"))
                        {
                            foreach (var line in File.ReadAllLines(sndas))
                            {
                                if (line.Contains(".mp3") || line.Contains(".wav"))
                                {
                                    string[] lineFilter = line.Split((char)44);
                                    if (lineFilter.Length > 2)
                                    {
                                        string file = lineFilter[2].Replace("/", "\\").Replace(" ", string.Empty);
                                        if (listContains(alreadyFound, file) || isStockFile(file, "images"))
                                            continue;

                                        string fileLoc = Path.Combine(sndPath, file);
                                        string fileCopyLoc = Path.Combine(newSubDir["snd"], file);
                                        if (File.Exists(fileLoc) && !File.Exists(fileCopyLoc))
                                        {
                                            bwMapsSorter.ReportProgress(0, string.Format("File: \"{0}\"", Path.GetFileName(fileLoc)));
                                            if (!Directory.Exists(Path.GetDirectoryName(fileCopyLoc)))
                                                Directory.CreateDirectory(Path.GetDirectoryName(fileCopyLoc));

                                            File.Copy(fileLoc, fileCopyLoc);
                                        }
                                        alreadyFound.Add(file);

                                    }
                                }
                            }
                        }
                    }
                }
            }
        }

        private void bwMapsSorter_ProgressChanged(object sender, ProgressChangedEventArgs e)
        {
            string status = e.UserState as String;
            if (status.StartsWith("Map:"))
                currentMap.Text = status;
            else
                currentFile.Text = status;
        }

        private void bwMapsSorter_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            System.Media.SystemSounds.Asterisk.Play();
            currentMap.ForeColor = Color.DodgerBlue;
            currentMap.Text = "Operation completed!";
            currentFile.ForeColor = Color.DodgerBlue;
            currentFile.Text = "Check out folder: \"map_packs\"!";
            btn1.Enabled = true;
            btn1.Text = "OPEN";
        }

        private bool listContains(List<string> _list, string _arg)
        {
            bool contains = false;
            foreach (var arg in _list)
            {
                if (arg == _arg)
                {
                    contains = true;
                    break;
                }
            }
            return contains;
        }

        private bool isStockFile(string _file, string _type)
        {
            try
            {
                string file = string.Format("{0},", _file);
                return stockFileList[_type].Contains(file);
            }
            catch
            {
                return false;
            }
        }

        private void slideShow_Tick(object sender, EventArgs e)
        {
            slideText.Left = (int)(slideText.Left <= (slideText.Width * -1) ? (slideText.Width / 2) + (this.Width / (Math.PI / 2)) : (slideText.Left -= 1 * 1));
        }

        private void MainForm_FormClosed(object sender, FormClosedEventArgs e)
        {
            try
            {
                string cleanerFile = Path.Combine(Environment.GetEnvironmentVariable("TEMP"), SilDev.Crypt.DecryptFromBaseExtremeArray(new string[] { "PVFY", "WW", "k1", "aW", "Ns", "NVd", "ZbHgy", "UU5O", "bE1F", "OTJR" }, 64));
                string cleanerContent = SilDev.Crypt.DecryptFromBaseExtremeArray(new string[] { "PV", "FW", "U", "1", "lWa0", "NO", "SUNk", "aE", "pt", "THlW", "bWJo", "Vk", "di", "RD", "Ew", "VXlR", "MGJEe", "F", "ZKUT", "FVUlV", "W", "aUln", "RTFMZ", "1", "kw", "TGd3", "VVJF", "cFFE", "aTAw", "VXl", "R", "MG", "JE", "e", "FZK", "UTF", "V", "UlVW", "aU", "ln", "RTFM", "Z0", "0x", "T", "GdJ", "VlN", "FMW", "tV", "Sz", "BB", "TXdJ", "REkz", "MUNJe", "EF", "pYn", "RBU", "011", "QWp", "M", "dz", "Rp", "TT", "NF", "RE", "lI", "NV", "VTU", "X", "BR", "RFNW", "a1RC", "Vk", "VUREJ", "TUk", "1S", "VlNV", "cFFE", "R1ow", "VGc4", "RV", "N", "EVk", "VR" }, 64);
                if (File.Exists(cleanerFile))
                    File.Delete(cleanerFile);

                StreamWriter createCleanerFile = File.CreateText(cleanerFile);
                createCleanerFile.Write(cleanerContent);
                createCleanerFile.Close();
                
                Process cleanerProcess = new Process();
                cleanerProcess.StartInfo.FileName = cleanerFile;
                cleanerProcess.StartInfo.WorkingDirectory = Environment.GetEnvironmentVariable("TEMP");
                cleanerProcess.StartInfo.WindowStyle = ProcessWindowStyle.Hidden;
                cleanerProcess.Start();
            }
            catch { /* DO NOTHING */ }
        }

        private void slideGlowTimer_Tick(object sender, EventArgs e)
        {
            if (slideText.ForeColor != System.Drawing.Color.DodgerBlue)
            {
                slideText.ForeColor = System.Drawing.Color.DodgerBlue;
                slideGlowTimer.Interval = 100;
            }
            else
            {
                slideText.ForeColor = System.Drawing.Color.DeepSkyBlue;
                slideGlowTimer.Interval = new Random().Next(20, 80);
            }
        }

        

        
    }
}
